@@ -1,10 +1,10 @@
-# Copyright 2018-2022 Streamlit Inc.
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#    http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
@@ -18,18 +18,15 @@ import copy
 import os
 import secrets
 import threading
-import toml
 from collections import OrderedDict
-from typing import Any, Callable, cast, Dict, Optional
+from typing import Any, Callable, Dict, Optional, cast
 
+import toml
 from blinker import Signal
 
-from streamlit import config_util
-from streamlit import development
-from streamlit import env_util
-from streamlit import file_util
-from streamlit import util
+from streamlit import config_util, development, env_util, file_util, util
 from streamlit.config_option import ConfigOption
+from streamlit.errors import StreamlitAPIException
 
 # Config System Global State #
 
@@ -68,6 +65,9 @@ def set_option(key: str, value: Any, where_defined: str = _USER_DEFINED) -> None
 
     Run `streamlit config show` in the terminal to see all available options.
 
+    This is an internal API. The public `st.set_option` API is implemented
+    in `set_user_option`.
+
     Parameters
     ----------
     key : str
@@ -80,13 +80,51 @@ def set_option(key: str, value: Any, where_defined: str = _USER_DEFINED) -> None
     where_defined : str
         Tells the config system where this was set.
     """
-
     with _config_lock:
         # Ensure that our config files have been parsed.
         get_config_options()
         _set_option(key, value, where_defined)
 
 
+def set_user_option(key: str, value: Any) -> None:
+    """Set config option.
+
+    Currently, only the following config options can be set within the script itself:
+        * client.caching
+        * client.displayEnabled
+        * deprecation.*
+
+    Calling with any other options will raise StreamlitAPIException.
+
+    Run `streamlit config show` in the terminal to see all available options.
+
+    Parameters
+    ----------
+    key : str
+        The config option key of the form "section.optionName". To see all
+        available options, run `streamlit config show` on a terminal.
+
+    value
+        The new value to assign to this config option.
+
+    """
+    try:
+        opt = _config_options_template[key]
+    except KeyError as ke:
+        raise StreamlitAPIException(
+            "Unrecognized config option: {key}".format(key=key)
+        ) from ke
+    if opt.scriptable:
+        set_option(key, value)
+        return
+
+    raise StreamlitAPIException(
+        "{key} cannot be set on the fly. Set as command line option, e.g. streamlit run script.py --{key}, or in config.toml instead.".format(
+            key=key
+        )
+    )
+
+
 def get_option(key: str) -> Any:
     """Return the current value of a given Streamlit config option.
 
@@ -117,7 +155,7 @@ def get_options_for_section(section: str) -> Dict[str, Any]:
         The name of the config section to fetch options for.
 
     Returns
-    ----------
+    -------
     Dict[str, Any]
         A dict mapping the names of the options in the given section (without
         the section name as a prefix) to their values.
@@ -219,6 +257,7 @@ def _delete_option(key: str) -> None:
         del _config_options_template[key]
         del cast(Dict[str, ConfigOption], _config_options)[key]
     except Exception:
+        # We don't care if the option already doesn't exist.
         pass
 
 
@@ -279,15 +318,13 @@ _create_option(
     replaced_by="logger.level",
 )
 
-
-@_create_option("global.unitTest", visibility="hidden", type_=bool)
-def _global_unit_test() -> bool:
-    """Are we in a unit test?
-
-    This option defaults to False.
-    """
-    return False
-
+_create_option(
+    "global.unitTest",
+    description="Are we in a unit test?",
+    visibility="hidden",
+    default_val=False,
+    type_=bool,
+)
 
 _create_option(
     "global.suppressDeprecationWarnings",
@@ -340,7 +377,6 @@ def _logger_log_level() -> str:
 
     Default: 'info'
     """
-
     if get_option("global.logLevel"):
         return str(get_option("global.logLevel"))
     elif get_option("global.developmentMode"):
@@ -405,13 +441,14 @@ _create_option(
 _create_option(
     "client.showErrorDetails",
     description="""
-        Controls whether uncaught app exceptions are displayed in the browser.
-        By default, this is set to True and Streamlit displays app exceptions
-        and associated tracebacks in the browser.
+        Controls whether uncaught app exceptions and deprecation warnings
+        are displayed in the browser. By default, this is set to True and
+        Streamlit displays app exceptions and associated tracebacks, and
+        deprecation warnings, in the browser.
 
-        If set to False, an exception will result in a generic message being
-        shown in the browser, and exceptions and tracebacks will be printed to
-        the console only.""",
+        If set to False, an exception or deprecation warning will result in
+        a generic message being shown in the browser, and exceptions, tracebacks,
+        and deprecation warnings will be printed to the console only.""",
     default_val=True,
     type_=bool,
     scriptable=True,
@@ -540,22 +577,24 @@ def _server_headless() -> bool:
     return False
 
 
-@_create_option("server.runOnSave", type_=bool)
-def _server_run_on_save() -> bool:
-    """Automatically rerun script when the file is modified on disk.
+_create_option(
+    "server.runOnSave",
+    description="""
+        Automatically rerun script when the file is modified on disk.
+        """,
+    default_val=False,
+    type_=bool,
+)
 
-    Default: false
-    """
-    return False
-
-
-@_create_option("server.allowRunOnSave", type_=bool, visibility="hidden")
-def _server_allow_run_on_save() -> bool:
-    """Allows users to automatically rerun when app is updated.
-
-    Default: true
-    """
-    return True
+_create_option(
+    "server.allowRunOnSave",
+    description="""
+        Allows users to automatically rerun when app is updated.
+        """,
+    visibility="hidden",
+    default_val=True,
+    type_=bool,
+)
 
 
 @_create_option("server.address")
@@ -570,15 +609,13 @@ def _server_address() -> Optional[str]:
     return None
 
 
-@_create_option("server.port", type_=int)
-def _server_port() -> int:
-    """The port where the server will listen for browser
-    connections.
-
-    Default: 8501
-    """
-    return 8501
-
+_create_option(
+    "server.port",
+    description="""
+        The port where the server will listen for browser connections.""",
+    default_val=8501,
+    type_=int,
+)
 
 _create_option(
     "server.scriptHealthCheckEnabled",
@@ -604,88 +641,97 @@ _create_option(
     type_=str,
 )
 
-
 # TODO: Rename to server.enableCorsProtection.
-@_create_option("server.enableCORS", type_=bool)
-def _server_enable_cors() -> bool:
-    """Enables support for Cross-Origin Request Sharing (CORS) protection, for added security.
+_create_option(
+    "server.enableCORS",
+    description="""
+    Enables support for Cross-Origin Resource Sharing (CORS) protection, for added security.
 
     Due to conflicts between CORS and XSRF, if `server.enableXsrfProtection` is on and
     `server.enableCORS` is off at the same time, we will prioritize `server.enableXsrfProtection`.
-
-    Default: true
-    """
-    return True
+    """,
+    default_val=True,
+    type_=bool,
+)
 
 
-@_create_option("server.enableXsrfProtection", type_=bool)
-def _server_enable_xsrf_protection() -> bool:
-    """Enables support for Cross-Site Request Forgery (XSRF) protection, for added security.
+_create_option(
+    "server.enableXsrfProtection",
+    description="""
+        Enables support for Cross-Site Request Forgery (XSRF) protection, for added security.
 
-    Due to conflicts between CORS and XSRF, if `server.enableXsrfProtection` is on and
-    `server.enableCORS` is off at the same time, we will prioritize `server.enableXsrfProtection`.
+        Due to conflicts between CORS and XSRF, if `server.enableXsrfProtection` is on and
+        `server.enableCORS` is off at the same time, we will prioritize `server.enableXsrfProtection`.
+        """,
+    default_val=True,
+    type_=bool,
+)
 
-    Default: true
-    """
-    return True
+_create_option(
+    "server.maxUploadSize",
+    description="""
+        Max size, in megabytes, for files uploaded with the file_uploader.
+        """,
+    default_val=200,  # If this default is changed, please also update the docstring for `DeltaGenerator.file_uploader`.
+    type_=int,
+)
 
+_create_option(
+    "server.maxMessageSize",
+    description="""
+        Max size, in megabytes, of messages that can be sent via the WebSocket connection.
+        """,
+    default_val=200,
+    type_=int,
+)
 
-@_create_option("server.maxUploadSize", type_=int)
-def _server_max_upload_size() -> int:
-    """Max size, in megabytes, for files uploaded with the file_uploader.
-
-    Default: 200
-    """
-    # If this default is changed, please also update the docstring
-    # for `DeltaGenerator.file_uploader`.
-    return 200
-
-
-@_create_option("server.maxMessageSize", type_=int)
-def _server_max_message_size() -> int:
-    """Max size, in megabytes, of messages that can be sent via the WebSocket connection.
-
-    Default: 200
-    """
-    return 200
-
-
-@_create_option("server.enableWebsocketCompression", type_=bool)
-def _server_enable_websocket_compression() -> bool:
-    """Enables support for websocket compression.
-
-    Default: false
-    """
-    return False
+_create_option(
+    "server.enableWebsocketCompression",
+    description="""
+        Enables support for websocket compression.
+        """,
+    default_val=False,
+    type_=bool,
+)
 
+_create_option(
+    "server.enableStaticServing",
+    description="""
+        Enable serving files from a `static` directory in the running app's directory.
+        """,
+    default_val=False,
+    type_=bool,
+)
 
 # Config Section: Browser #
 
 _create_section("browser", "Configuration of non-UI browser options.")
 
 
-@_create_option("browser.serverAddress")
-def _browser_server_address() -> str:
-    """Internet address where users should point their browsers in order to
-    connect to the app. Can be IP address or DNS name and path.
+_create_option(
+    "browser.serverAddress",
+    description="""
+        Internet address where users should point their browsers in order to
+        connect to the app. Can be IP address or DNS name and path.
 
-    This is used to:
-    - Set the correct URL for CORS and XSRF protection purposes.
-    - Show the URL on the terminal
-    - Open the browser
-
-    Default: 'localhost'
-    """
-    return "localhost"
+        This is used to:
+        - Set the correct URL for CORS and XSRF protection purposes.
+        - Show the URL on the terminal
+        - Open the browser
+        """,
+    default_val="localhost",
+    type_=str,
+)
 
 
-@_create_option("browser.gatherUsageStats", type_=bool)
-def _gather_usage_stats() -> bool:
-    """Whether to send usage statistics to Streamlit.
-
-    Default: true
-    """
-    return True
+_create_option(
+    "browser.gatherUsageStats",
+    description="""
+        Whether to send usage statistics to Streamlit.
+        """,
+    default_val=True,
+    type_=bool,
+)
 
 
 @_create_option("browser.serverPort", type_=int)
@@ -956,7 +1002,6 @@ def _maybe_read_env_variable(value: Any) -> Any:
         variable.
 
     """
-
     if isinstance(value, str) and value.startswith("env:"):
         var_name = value[len("env:") :]
         env_var = os.environ.get(var_name)
@@ -1022,7 +1067,7 @@ def get_config_options(
         Config options that we received via CLI flag.
 
     Returns
-    ----------
+    -------
     Dict[str, ConfigOption]
         An ordered dict that maps config option names to their values.
     """
@@ -1144,7 +1189,6 @@ def on_config_parsed(
     Callable[[], bool]
         A function that the caller can use to deregister func.
     """
-
     # We need to use the same receiver when we connect or disconnect on the
     # Signal. If we don't do this, then the registered receiver won't be released
     # leading to a memory leak because the Signal will keep a reference of the
