@@ -1,10 +1,10 @@
-# Copyright 2018-2022 Streamlit Inc.
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#    http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
@@ -12,22 +12,24 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import cast, Optional, TYPE_CHECKING
+from typing import TYPE_CHECKING, Optional, cast
 
 from streamlit.proto.IFrame_pb2 import IFrame as IFrameProto
+from streamlit.runtime.metrics_util import gather_metrics
 
 if TYPE_CHECKING:
     from streamlit.delta_generator import DeltaGenerator
 
 
 class IframeMixin:
+    @gather_metrics("_iframe")
     def _iframe(
         self,
         src: str,
         width: Optional[int] = None,
         height: Optional[int] = None,
         scrolling: bool = False,
-    ):
+    ) -> "DeltaGenerator":
         """Load a remote URL in an iframe.
 
         Parameters
@@ -54,13 +56,14 @@ class IframeMixin:
         )
         return self.dg._enqueue("iframe", iframe_proto)
 
+    @gather_metrics("_html")
     def _html(
         self,
         html: str,
         width: Optional[int] = None,
         height: Optional[int] = None,
         scrolling: bool = False,
-    ):
+    ) -> "DeltaGenerator":
         """Display an HTML string in an iframe.
 
         Parameters
