@@ -1,10 +1,10 @@
-# Copyright 2018-2022 Streamlit Inc.
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#    http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
@@ -13,36 +13,29 @@
 # limitations under the License.
 
 import json
-import copy
-from typing import Any, cast, TYPE_CHECKING
+from typing import TYPE_CHECKING, Any, List, Union, cast
 
 from streamlit.proto.Json_pb2 import Json as JsonProto
+from streamlit.runtime.metrics_util import gather_metrics
 from streamlit.runtime.state import SessionStateProxy
 from streamlit.user_info import UserInfoProxy
 
-
 if TYPE_CHECKING:
     from streamlit.delta_generator import DeltaGenerator
 
 
-def _convert_sets_to_lists(body: Any) -> Any:
-    # Convert sets into lists, which render more nicely on the frontend
-    set_found = False
-    for key in body:
-        if isinstance(body[key], set):
-            if not set_found:
-                # When set is found to prevent mutation of input body, we need to shallow copy it once
-                # To avoid copying it multiple times we use set_found flag
-                body = copy.copy(body)
-                set_found = True
-            body[key] = list(body[key])
-    return body
+def _ensure_serialization(o: object) -> Union[str, List[Any]]:
+    """A repr function for json.dumps default arg, which tries to serialize sets as lists"""
+    if isinstance(o, set):
+        return list(o)
+    return repr(o)
 
 
 class JsonMixin:
+    @gather_metrics("json")
     def json(
         self,
-        body: Any,
+        body: object,
         *,  # keyword-only arguments:
         expanded: bool = True,
     ) -> "DeltaGenerator":
@@ -50,7 +43,7 @@ class JsonMixin:
 
         Parameters
         ----------
-        body : Object or str
+        body : object or str
             The object to print as JSON. All referenced objects should be
             serializable to JSON as well. If object is a string, we assume it
             contains serialized JSON.
@@ -62,6 +55,8 @@ class JsonMixin:
 
         Example
         -------
+        >>> import streamlit as st
+        >>>
         >>> st.json({
         ...     'foo': 'bar',
         ...     'baz': 'boz',
@@ -84,24 +79,15 @@ class JsonMixin:
             body = body.to_dict()
 
         if not isinstance(body, str):
-            # Check if body is iterable, if it is convert it's sets to lists
             try:
-                iter(body)
-            except TypeError:
-                # body is not iterable, do nothing with the body
-                pass
-            else:
-                # body is iterable, look for sets and change them to lists
-                body = _convert_sets_to_lists(body)
-            try:
-                # Serialize body to string
-                body = json.dumps(body, default=repr)
+                # Serialize body to string and try to interpret sets as lists
+                body = json.dumps(body, default=_ensure_serialization)
             except TypeError as err:
                 st.warning(
                     "Warning: this data structure was not fully serializable as "
-                    "JSON due to one or more unexpected keys.  (Error was: %s)" % err
+                    f"JSON due to one or more unexpected keys.  (Error was: {err})"
                 )
-                body = json.dumps(body, skipkeys=True, default=repr)
+                body = json.dumps(body, skipkeys=True, default=_ensure_serialization)
 
         json_proto = JsonProto()
         json_proto.body = body
