@@ -1,10 +1,10 @@
-# Copyright 2018-2022 Streamlit Inc.
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#    http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
@@ -16,29 +16,40 @@
 
 import datetime
 import re
-from collections import namedtuple
-from typing import cast, Dict, Any, Optional
+from typing import TYPE_CHECKING, Any, Dict, NamedTuple, Optional, cast
 
 import pyarrow as pa
 import tzlocal
 from pandas import DataFrame
 from pandas.io.formats.style import Styler
+from typing_extensions import Final
 
-import streamlit
 from streamlit import errors, type_util
+from streamlit.elements.arrow import Data
 from streamlit.logger import get_logger
-from streamlit.proto.DataFrame_pb2 import (
-    DataFrame as DataFrameProto,
-    TableStyle as TableStyleProto,
-)
+from streamlit.proto.DataFrame_pb2 import DataFrame as DataFrameProto
+from streamlit.proto.DataFrame_pb2 import TableStyle as TableStyleProto
+from streamlit.runtime.metrics_util import gather_metrics
 
-LOGGER = get_logger(__name__)
+if TYPE_CHECKING:
+    from streamlit.delta_generator import DeltaGenerator
 
-CSSStyle = namedtuple("CSSStyle", ["property", "value"])
+LOGGER: Final = get_logger(__name__)
+
+
+class CSSStyle(NamedTuple):
+    property: Any
+    value: Any
 
 
 class LegacyDataFrameMixin:
-    def _legacy_dataframe(self, data=None, width=None, height=None):
+    @gather_metrics("_legacy_dataframe")
+    def _legacy_dataframe(
+        self,
+        data: Data = None,
+        width: Optional[int] = None,
+        height: Optional[int] = None,
+    ) -> "DeltaGenerator":
         """Display a dataframe as an interactive table.
 
         Parameters
@@ -48,7 +59,7 @@ class LegacyDataFrameMixin:
             The data to display.
 
             If 'data' is a pandas.Styler, it will be used to style its
-            underyling DataFrame. Streamlit supports custom cell
+            underlying DataFrame. Streamlit supports custom cell
             values and colors. (It does not support some of the more exotic
             pandas styling features, like bar charts, hovering, and captions.)
             Styler support is experimental!
@@ -61,6 +72,10 @@ class LegacyDataFrameMixin:
 
         Examples
         --------
+        >>> import streamlit as st
+        >>> import pandas as pd
+        >>> import numpy as np
+        >>>
         >>> df = pd.DataFrame(
         ...    np.random.randn(50, 20),
         ...    columns=('col %d' % i for i in range(20)))
@@ -76,6 +91,10 @@ class LegacyDataFrameMixin:
         You can also pass a Pandas Styler object to change the style of
         the rendered DataFrame:
 
+        >>> import streamlit as st
+        >>> import pandas as pd
+        >>> import numpy as np
+        >>>
         >>> df = pd.DataFrame(
         ...    np.random.randn(10, 20),
         ...    columns=('col %d' % i for i in range(20)))
@@ -97,7 +116,8 @@ class LegacyDataFrameMixin:
             element_height=height,
         )
 
-    def _legacy_table(self, data=None):
+    @gather_metrics("_legacy_table")
+    def _legacy_table(self, data: Data = None) -> "DeltaGenerator":
         """Display a static table.
 
         This differs from `st._legacy_dataframe` in that the table in this case is
@@ -111,6 +131,10 @@ class LegacyDataFrameMixin:
 
         Example
         -------
+        >>> import streamlit as st
+        >>> import pandas as pd
+        >>> import numpy as np
+        >>>
         >>> df = pd.DataFrame(
         ...    np.random.randn(10, 5),
         ...    columns=('col %d' % i for i in range(5)))
@@ -127,12 +151,12 @@ class LegacyDataFrameMixin:
         return self.dg._enqueue("table", table_proto)
 
     @property
-    def dg(self) -> "streamlit.delta_generator.DeltaGenerator":
+    def dg(self) -> "DeltaGenerator":
         """Get our DeltaGenerator."""
-        return cast("streamlit.delta_generator.DeltaGenerator", self)
+        return cast("DeltaGenerator", self)
 
 
-def marshall_data_frame(data: Any, proto_df: DataFrameProto) -> None:
+def marshall_data_frame(data: Data, proto_df: DataFrameProto) -> None:
     """Convert a pandas.DataFrame into a proto.DataFrame.
 
     Parameters
@@ -338,14 +362,14 @@ def _get_custom_display_values(translated_style: Dict[Any, Any]) -> Dict[Any, An
     return display_values
 
 
-def _marshall_index(pandas_index, proto_index):
+def _marshall_index(pandas_index, proto_index) -> None:
     """Convert an pandas.Index into a proto.Index.
 
     pandas_index - Panda.Index or related (input)
     proto_index  - proto.Index (output)
     """
-    import pandas as pd
     import numpy as np
+    import pandas as pd
 
     if type(pandas_index) == pd.Index:
         _marshall_any_array(np.array(pandas_index), proto_index.plain_index.data)
@@ -385,7 +409,7 @@ def _marshall_index(pandas_index, proto_index):
         raise NotImplementedError("Can't handle %s yet." % type(pandas_index))
 
 
-def _marshall_table(pandas_table, proto_table):
+def _marshall_table(pandas_table, proto_table) -> None:
     """Convert a sequence of 1D arrays into proto.Table.
 
     pandas_table - Sequence of 1D arrays which are AnyArray compatible (input).
@@ -397,7 +421,7 @@ def _marshall_table(pandas_table, proto_table):
         _marshall_any_array(pandas_array, proto_table.cols.add())
 
 
-def _marshall_any_array(pandas_array, proto_array):
+def _marshall_any_array(pandas_array, proto_array) -> None:
     """Convert a 1D numpy.Array into a proto.AnyArray.
 
     pandas_array - 1D arrays which is AnyArray compatible (input).
@@ -439,180 +463,3 @@ def _marshall_any_array(pandas_array, proto_array):
         proto_array.datetimes.data.extend(pandas_array.map(datetime.datetime.isoformat))
     else:
         raise NotImplementedError("Dtype %s not understood." % pandas_array.dtype)
-
-
-def add_rows(delta1, delta2, name=None):
-    """Concat the DataFrame in delta2 to the DataFrame in delta1.
-
-    Parameters
-    ----------
-    delta1 : Delta
-    delta2 : Delta
-    name : str or None
-
-    """
-    df1 = _get_data_frame(delta1, name)
-    df2 = _get_data_frame(delta2, name)
-
-    if len(df1.data.cols) == 0:
-        if len(df2.data.cols) == 0:
-            return
-        df1.CopyFrom(df2)
-        return
-
-    # Copy Data
-    if len(df1.data.cols) != len(df2.data.cols):
-        raise ValueError("Dataframes have incompatible shapes")
-    for (col1, col2) in zip(df1.data.cols, df2.data.cols):
-        _concat_any_array(col1, col2)
-
-    # Copy index
-    _concat_index(df1.index, df2.index)
-
-    # Don't concat columns! add_rows should leave the dataframe with the same
-    # number of columns as it had before.
-    # DON'T DO: _concat_index(df1.columns, df2.columns)
-
-    # Copy styles
-    for (style_col1, style_col2) in zip(df1.style.cols, df2.style.cols):
-        _concat_cell_style_array(style_col1, style_col2)
-
-
-def _concat_index(index1, index2):
-    """Contact index2 into index1."""
-    # Special case if index1 is empty.
-    if _index_len(index1) == 0:
-        index1.Clear()
-        index1.CopyFrom(index2)
-        return
-
-    # Otherwise, dispatch based on type.
-    type1 = index1.WhichOneof("type")
-    type2 = index2.WhichOneof("type")
-    # This branch is covered with tests but pytest doesnt seem to realize it.
-    if type1 != type2:  # pragma: no cover
-        raise ValueError(
-            "Cannot concatenate %(type1)s with %(type2)s."
-            % {"type1": type1, "type2": type2}
-        )
-
-    if type1 == "plain_index":
-        _concat_any_array(index1.plain_index.data, index2.plain_index.data)
-    elif type1 == "range_index":
-        index1.range_index.stop += index2.range_index.stop - index2.range_index.start
-    elif type1 == "multi_index":
-        raise NotImplementedError("Cannot yet concatenate MultiIndices.")
-    elif type1 == "int_64_index":
-        index1.int_64_index.data.data.extend(index2.int_64_index.data.data)
-    elif type1 == "datetime_index":
-        index1.datetime_index.data.data.extend(index2.datetime_index.data.data)
-    elif type1 == "timedelta_index":
-        index1.timedelta_index.data.data.extend(index2.timedelta_index.data.data)
-    else:
-        raise NotImplementedError('Cannot concatenate "%s" indices.' % type1)
-
-
-def _concat_any_array(any_array_1, any_array_2):
-    """Concat elements from any_array_2 into any_array_1."""
-    # Special case if any_array_1 is empty
-    if _any_array_len(any_array_1) == 0:
-        any_array_1.CopyFrom(any_array_2)
-        return
-
-    type1 = any_array_1.WhichOneof("type")
-    type2 = any_array_2.WhichOneof("type")
-    if type1 != type2:
-        raise ValueError(
-            "Cannot concatenate %(type1)s with %(type2)s."
-            % {"type1": type1, "type2": type2}
-        )
-    getattr(any_array_1, type1).data.extend(getattr(any_array_2, type2).data)
-
-
-def _concat_cell_style_array(style_array1, style_array2):
-    """Concat elements from any_array_2 into any_array_1."""
-    # Special case if array1 is empty
-    if len(style_array1.styles) == 0:
-        style_array1.CopyFrom(style_array2)
-        return
-
-    style_array1.styles.extend(style_array2.styles)
-
-
-def _get_data_frame(delta, name=None):
-    """Extract the dataframe from a delta."""
-    delta_type = delta.WhichOneof("type")
-
-    if delta_type == "new_element":
-        element_type = delta.new_element.WhichOneof("type")
-
-        # Some element types don't support named datasets.
-        if name and element_type in ("data_frame", "table", "chart"):
-            raise ValueError("Dataset names not supported for st.%s" % element_type)
-
-        if element_type in "data_frame":
-            return delta.new_element.data_frame
-        elif element_type in "table":
-            return delta.new_element.table
-        elif element_type == "chart":
-            return delta.new_element.chart.data
-        elif element_type == "vega_lite_chart":
-            chart_proto = delta.new_element.vega_lite_chart
-            if name:
-                return _get_or_create_dataset(chart_proto.datasets, name)
-            elif len(chart_proto.datasets) == 1:
-                # Support the case where the dataset name was randomly given by
-                # the charting library (e.g. Altair) and the user has no
-                # knowledge of it.
-                return chart_proto.datasets[0].data
-            else:
-                return chart_proto.data
-        # TODO: Support DeckGL. Need to figure out how to handle layer indices
-        # first.
-
-    elif delta_type == "add_rows":
-        if delta.add_rows.has_name and name != delta.add_rows.name:
-            raise ValueError('No dataset found with name "%s".' % name)
-        return delta.add_rows.data
-    else:
-        raise ValueError("Cannot extract DataFrame from %s." % delta_type)
-
-
-def _get_or_create_dataset(datasets_proto, name):
-    for dataset in datasets_proto:
-        if dataset.has_name and dataset.name == name:
-            return dataset.data
-
-    dataset = datasets_proto.add()
-    dataset.name = name
-    dataset.has_name = True
-    return dataset.data
-
-
-def _index_len(index):
-    """Return the number of elements in an index."""
-    index_type = index.WhichOneof("type")
-    if index_type == "plain_index":
-        return _any_array_len(index.plain_index.data)
-    elif index_type == "range_index":
-        return index.range_index.stop - index.range_index.start
-    elif index_type == "multi_index":
-        if len(index.multi_index.labels) == 0:
-            return 0
-        else:
-            return len(index.multi_index.labels[0].data)
-    elif index_type == "int_64_index":
-        return len(index.int_64_index.data.data)
-    elif index_type == "float_64_index":
-        return len(index.float_64_index.data.data)
-    elif index_type == "datetime_index":
-        return len(index.datetime_index.data.data)
-    elif index_type == "timedelta_index":
-        return len(index.timedelta_index.data.data)
-
-
-def _any_array_len(any_array):
-    """Return the length of an any_array."""
-    array_type = any_array.WhichOneof("type")
-    the_array = getattr(any_array, array_type).data
-    return len(the_array)
