@@ -1,10 +1,10 @@
-# Copyright 2018-2022 Streamlit Inc.
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#    http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
@@ -14,15 +14,18 @@
 
 import io
 import re
-from typing import cast, Optional, TYPE_CHECKING, Union
-from typing_extensions import Final, TypeAlias
+from typing import TYPE_CHECKING, Optional, Tuple, Union, cast
 
+from typing_extensions import Final, TypeAlias
 from validators import url
 
-from streamlit import type_util
-from streamlit.runtime.in_memory_file_manager import in_memory_file_manager
+import streamlit as st
+from streamlit import runtime, type_util
+from streamlit.errors import StreamlitAPIException
 from streamlit.proto.Audio_pb2 import Audio as AudioProto
 from streamlit.proto.Video_pb2 import Video as VideoProto
+from streamlit.runtime import caching
+from streamlit.runtime.metrics_util import gather_metrics
 
 if TYPE_CHECKING:
     from typing import Any
@@ -31,18 +34,20 @@ if TYPE_CHECKING:
 
     from streamlit.delta_generator import DeltaGenerator
 
-
-Data: TypeAlias = Union[
+MediaData: TypeAlias = Union[
     str, bytes, io.BytesIO, io.RawIOBase, io.BufferedReader, "npt.NDArray[Any]", None
 ]
 
 
 class MediaMixin:
+    @gather_metrics("audio")
     def audio(
         self,
-        data: Data,
+        data: MediaData,
         format: str = "audio/wav",
         start_time: int = 0,
+        *,
+        sample_rate: Optional[int] = None,
     ) -> "DeltaGenerator":
         """Display an audio player.
 
@@ -51,34 +56,68 @@ class MediaMixin:
         data : str, bytes, BytesIO, numpy.ndarray, or file opened with
                 io.open().
             Raw audio data, filename, or a URL pointing to the file to load.
-            Numpy arrays and raw data formats must include all necessary file
-            headers to match specified file format.
-        start_time: int
-            The time from which this element should start playing.
+            Raw data formats must include all necessary file headers to match the file
+            format specified via ``format``.
+            If ``data`` is a numpy array, it must either be a 1D array of the waveform
+            or a 2D array of shape ``(num_channels, num_samples)`` with waveforms
+            for all channels. See the default channel order at
+            http://msdn.microsoft.com/en-us/library/windows/hardware/dn653308(v=vs.85).aspx
         format : str
             The mime type for the audio file. Defaults to 'audio/wav'.
             See https://tools.ietf.org/html/rfc4281 for more info.
+        start_time: int
+            The time from which this element should start playing.
+        sample_rate: int or None
+            The sample rate of the audio data in samples per second. Only required if
+            ``data`` is a numpy array.
 
         Example
         -------
+        >>> import streamlit as st
+        >>> import numpy as np
+        >>>
         >>> audio_file = open('myaudio.ogg', 'rb')
         >>> audio_bytes = audio_file.read()
         >>>
         >>> st.audio(audio_bytes, format='audio/ogg')
+        >>>
+        >>> sample_rate = 44100  # 44100 samples per second
+        >>> seconds = 2  # Note duration of 2 seconds
+        >>> frequency_la = 440  # Our played note will be 440 Hz
+        >>> # Generate array with seconds*sample_rate steps, ranging between 0 and seconds
+        >>> t = np.linspace(0, seconds, seconds * sample_rate, False)
+        >>> # Generate a 440 Hz sine wave
+        >>> note_la = np.sin(frequency_la * t * 2 * np.pi)
+        >>>
+        >>> st.audio(note_la, sample_rate=sample_rate)
 
         .. output::
            https://doc-audio.streamlitapp.com/
-           height: 465px
+           height: 865px
 
         """
         audio_proto = AudioProto()
         coordinates = self.dg._get_delta_path_str()
-        marshall_audio(coordinates, audio_proto, data, format, start_time)
+
+        is_data_numpy_array = type_util.is_type(data, "numpy.ndarray")
+
+        if is_data_numpy_array and sample_rate is None:
+            raise StreamlitAPIException(
+                "`sample_rate` must be specified when `data` is a numpy array."
+            )
+        if not is_data_numpy_array and sample_rate is not None:
+            st.warning(
+                "Warning: `sample_rate` will be ignored since data is not a numpy "
+                "array."
+            )
+
+        marshall_audio(coordinates, audio_proto, data, format, start_time, sample_rate)
         return self.dg._enqueue("audio", audio_proto)
 
+    @gather_metrics("video")
     def video(
         self,
-        data: Data,
+        data: MediaData,
         format: str = "video/mp4",
         start_time: int = 0,
     ) -> "DeltaGenerator":
@@ -100,6 +139,8 @@ class MediaMixin:
 
         Example
         -------
+        >>> import streamlit as st
+        >>>
         >>> video_file = open('myvideo.mp4', 'rb')
         >>> video_bytes = video_file.read()
         >>>
@@ -166,7 +207,7 @@ def _reshape_youtube_url(url: str) -> Optional[str]:
 def _marshall_av_media(
     coordinates: str,
     proto: Union[AudioProto, VideoProto],
-    data: Data,
+    data: MediaData,
     mimetype: str,
 ) -> None:
     """Fill audio or video proto based on contents of data.
@@ -175,46 +216,52 @@ def _marshall_av_media(
     Otherwise assume strings are filenames and let any OS errors raise.
 
     Load data either from file or through bytes-processing methods into a
-    InMemoryFile object.  Pack proto with generated Tornado-based URL.
+    MediaFile object.  Pack proto with generated Tornado-based URL.
+
+    (When running in "raw" mode, we won't actually load data into the
+    MediaFileManager, and we'll return an empty URL.)
     """
     # Audio and Video methods have already checked if this is a URL by this point.
 
-    if isinstance(data, str):
-        # Assume it's a filename or blank.  Allow OS-based file errors.
-        with open(data, "rb") as fh:
-            this_file = in_memory_file_manager.add(fh.read(), mimetype, coordinates)
-            proto.url = this_file.url
-            return
-
-    data_as_bytes: bytes
     if data is None:
         # Allow empty values so media players can be shown without media.
         return
-    elif isinstance(data, bytes):
-        data_as_bytes = data
+
+    data_or_filename: Union[bytes, str]
+    if isinstance(data, (str, bytes)):
+        # Pass strings and bytes through unchanged
+        data_or_filename = data
     elif isinstance(data, io.BytesIO):
         data.seek(0)
-        data_as_bytes = data.getvalue()
+        data_or_filename = data.getvalue()
     elif isinstance(data, io.RawIOBase) or isinstance(data, io.BufferedReader):
         data.seek(0)
         read_data = data.read()
         if read_data is None:
             return
         else:
-            data_as_bytes = read_data
+            data_or_filename = read_data
     elif type_util.is_type(data, "numpy.ndarray"):
-        data_as_bytes = data.tobytes()
+        data_or_filename = data.tobytes()
     else:
         raise RuntimeError("Invalid binary data format: %s" % type(data))
 
-    this_file = in_memory_file_manager.add(data_as_bytes, mimetype, coordinates)
-    proto.url = this_file.url
+    if runtime.exists():
+        file_url = runtime.get_instance().media_file_mgr.add(
+            data_or_filename, mimetype, coordinates
+        )
+        caching.save_media_data(data_or_filename, mimetype, coordinates)
+    else:
+        # When running in "raw mode", we can't access the MediaFileManager.
+        file_url = ""
+
+    proto.url = file_url
 
 
 def marshall_video(
     coordinates: str,
     proto: VideoProto,
-    data: Data,
+    data: MediaData,
     mimetype: str = "video/mp4",
     start_time: int = 0,
 ) -> None:
@@ -254,12 +301,92 @@ def marshall_video(
         _marshall_av_media(coordinates, proto, data, mimetype)
 
 
+def _validate_and_normalize(data: "npt.NDArray[Any]") -> Tuple[bytes, int]:
+    """Validates and normalizes numpy array data.
+    We validate numpy array shape (should be 1d or 2d)
+    We normalize input data to int16 [-32768, 32767] range.
+
+    Parameters
+    ----------
+    data : numpy array
+        numpy array to be validated and normalized
+
+    Returns
+    -------
+    Tuple of (bytes, int)
+        (bytes, nchan)
+        where
+         - bytes : bytes of normalized numpy array converted to int16
+         - nchan : number of channels for audio signal. 1 for mono, or 2 for stereo.
+    """
+    # we import numpy here locally to import it only when needed (when numpy array given
+    # to st.audio data)
+    import numpy as np
+
+    data = np.array(data, dtype=float)
+
+    if len(data.shape) == 1:
+        nchan = 1
+    elif len(data.shape) == 2:
+        # In wave files,channels are interleaved. E.g.,
+        # "L1R1L2R2..." for stereo. See
+        # http://msdn.microsoft.com/en-us/library/windows/hardware/dn653308(v=vs.85).aspx
+        # for channel ordering
+        nchan = data.shape[0]
+        data = data.T.ravel()
+    else:
+        raise StreamlitAPIException("Numpy array audio input must be a 1D or 2D array.")
+
+    if data.size == 0:
+        return data.astype(np.int16).tobytes(), nchan
+
+    max_abs_value = np.max(np.abs(data))
+    # 16-bit samples are stored as 2's-complement signed integers,
+    # ranging from -32768 to 32767.
+    # scaled_data is PCM 16 bit numpy array, that's why we multiply [-1, 1] float
+    # values to 32_767 == 2 ** 15 - 1.
+    np_array = (data / max_abs_value) * 32767
+    scaled_data = np_array.astype(np.int16)
+    return scaled_data.tobytes(), nchan
+
+
+def _make_wav(data: "npt.NDArray[Any]", sample_rate: int) -> bytes:
+    """
+    Transform a numpy array to a PCM bytestring
+    We use code from IPython display module to convert numpy array to wave bytes
+    https://github.com/ipython/ipython/blob/1015c392f3d50cf4ff3e9f29beede8c1abfdcb2a/IPython/lib/display.py#L146
+    """
+    # we import wave here locally to import it only when needed (when numpy array given
+    # to st.audio data)
+    import wave
+
+    scaled, nchan = _validate_and_normalize(data)
+
+    with io.BytesIO() as fp, wave.open(fp, mode="wb") as waveobj:
+        waveobj.setnchannels(nchan)
+        waveobj.setframerate(sample_rate)
+        waveobj.setsampwidth(2)
+        waveobj.setcomptype("NONE", "NONE")
+        waveobj.writeframes(scaled)
+        return fp.getvalue()
+
+
+def _maybe_convert_to_wav_bytes(
+    data: MediaData, sample_rate: Optional[int]
+) -> MediaData:
+    """Convert data to wav bytes if the data type is numpy array."""
+    if type_util.is_type(data, "numpy.ndarray") and sample_rate is not None:
+        data = _make_wav(cast("npt.NDArray[Any]", data), sample_rate)
+    return data
+
+
 def marshall_audio(
     coordinates: str,
     proto: AudioProto,
-    data: Data,
+    data: MediaData,
     mimetype: str = "audio/wav",
     start_time: int = 0,
+    sample_rate: Optional[int] = None,
 ) -> None:
     """Marshalls an audio proto, using data and url processors as needed.
 
@@ -277,6 +404,8 @@ def marshall_audio(
         See https://tools.ietf.org/html/rfc4281 for more info.
     start_time : int
         The time from which this element should start playing. (default: 0)
+    sample_rate: int or None
+        Optional param to provide sample_rate in case of numpy array
     """
 
     proto.start_time = start_time
@@ -285,4 +414,5 @@ def marshall_audio(
         proto.url = data
 
     else:
+        data = _maybe_convert_to_wav_bytes(data, sample_rate)
         _marshall_av_media(coordinates, proto, data, mimetype)
