@@ -1,10 +1,10 @@
-# Copyright 2018-2022 Streamlit Inc.
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#    http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
@@ -13,28 +13,57 @@
 # limitations under the License.
 
 import numbers
-from streamlit.script_run_context import ScriptRunContext, get_script_run_ctx
-from streamlit.type_util import Key, to_key
+from dataclasses import dataclass
 from textwrap import dedent
 from typing import Optional, Union, cast
 
 import streamlit
+from streamlit.elements.form import current_form_id
+from streamlit.elements.utils import (
+    check_callback_rules,
+    check_session_state_rules,
+    get_label_visibility_proto_value,
+)
 from streamlit.errors import StreamlitAPIException
 from streamlit.js_number import JSNumber, JSNumberBoundsException
 from streamlit.proto.NumberInput_pb2 import NumberInput as NumberInputProto
-from streamlit.state.widgets import register_widget, NoValue
-from streamlit.state.session_state import (
+from streamlit.runtime.metrics_util import gather_metrics
+from streamlit.runtime.scriptrunner import ScriptRunContext, get_script_run_ctx
+from streamlit.runtime.state import (
+    NoValue,
     WidgetArgs,
     WidgetCallback,
     WidgetKwargs,
+    register_widget,
 )
-from .form import current_form_id
-from .utils import check_callback_rules, check_session_state_rules
+from streamlit.type_util import Key, LabelVisibility, maybe_raise_label_warnings, to_key
 
 Number = Union[int, float]
 
 
+@dataclass
+class NumberInputSerde:
+    value: Union[int, float]
+    data_type: int
+
+    def serialize(self, v: Number) -> Number:
+        return v
+
+    def deserialize(self, ui_value: Optional[Number], widget_id: str = "") -> Number:
+        if ui_value is not None:
+            val = ui_value
+        else:
+            # Widget has not been used; fallback to the original value,
+            val = self.value
+
+        if self.data_type == NumberInputProto.INT:
+            val = int(val)
+
+        return val
+
+
 class NumberInputMixin:
+    @gather_metrics("number_input")
     def number_input(
         self,
         label: str,
@@ -50,13 +79,38 @@ class NumberInputMixin:
         kwargs: Optional[WidgetKwargs] = None,
         *,  # keyword-only arguments:
         disabled: bool = False,
+        label_visibility: LabelVisibility = "visible",
     ) -> Number:
-        """Display a numeric input widget.
+        r"""Display a numeric input widget.
 
         Parameters
         ----------
         label : str
             A short label explaining to the user what this input is for.
+            The label can optionally contain Markdown and supports the following
+            elements: Bold, Italics, Strikethroughs, Inline Code, Emojis, and Links.
+
+            This also supports:
+
+            * Emoji shortcodes, such as ``:+1:``  and ``:sunglasses:``.
+              For a list of all supported codes,
+              see https://share.streamlit.io/streamlit/emoji-shortcodes.
+
+            * LaTeX expressions, by wrapping them in "$" or "$$" (the "$$"
+              must be on their own lines). Supported LaTeX functions are listed
+              at https://katex.org/docs/supported.html.
+
+            * Colored text, using the syntax ``:color[text to be colored]``,
+              where ``color`` needs to be replaced with any of the following
+              supported colors: blue, green, orange, red, violet.
+
+            Unsupported elements are not displayed. Display unsupported elements
+            as literal characters by backslash-escaping them. E.g.
+            ``1\. Not an ordered list``.
+
+            For accessibility reasons, you should never set an empty label (label="")
+            but hide it with label_visibility if needed. In the future, we may disallow
+            empty labels by raising an exception.
         min_value : int or float or None
             The minimum permitted value.
             If None, there will be no minimum.
@@ -91,6 +145,11 @@ class NumberInputMixin:
             An optional boolean, which disables the number input if set to
             True. The default is False. This argument can only be supplied by
             keyword.
+        label_visibility : "visible" or "hidden" or "collapsed"
+            The visibility of the label. If "hidden", the label doesn't show but there
+            is still empty space for it above the widget (equivalent to label="").
+            If "collapsed", both the label and the space are removed. Default is
+            "visible". This argument can only be supplied by keyword.
 
         Returns
         -------
@@ -100,11 +159,13 @@ class NumberInputMixin:
 
         Example
         -------
+        >>> import streamlit as st
+        >>>
         >>> number = st.number_input('Insert a number')
         >>> st.write('The current number is ', number)
 
         .. output::
-           https://share.streamlit.io/streamlit/docs/main/python/api-examples-source/widget.number_input.py
+           https://doc-number-input.streamlitapp.com/
            height: 260px
 
         """
@@ -122,6 +183,7 @@ class NumberInputMixin:
             args=args,
             kwargs=kwargs,
             disabled=disabled,
+            label_visibility=label_visibility,
             ctx=ctx,
         )
 
@@ -140,6 +202,7 @@ class NumberInputMixin:
         kwargs: Optional[WidgetKwargs] = None,
         *,  # keyword-only arguments:
         disabled: bool = False,
+        label_visibility: LabelVisibility = "visible",
         ctx: Optional[ScriptRunContext] = None,
     ) -> Number:
         key = to_key(key)
@@ -147,7 +210,7 @@ class NumberInputMixin:
         check_session_state_rules(
             default_value=None if isinstance(value, NoValue) else value, key=key
         )
-
+        maybe_raise_label_warnings(label, label_visibility)
         # Ensure that all arguments are of the same type.
         number_input_args = [min_value, max_value, value, step]
 
@@ -233,9 +296,13 @@ class NumberInputMixin:
         try:
             if all_ints:
                 if min_value is not None:
-                    JSNumber.validate_int_bounds(min_value, "`min_value`")  # type: ignore
+                    JSNumber.validate_int_bounds(
+                        min_value, "`min_value`"  # type: ignore
+                    )
                 if max_value is not None:
-                    JSNumber.validate_int_bounds(max_value, "`max_value`")  # type: ignore
+                    JSNumber.validate_int_bounds(
+                        max_value, "`max_value`"  # type: ignore
+                    )
                 if step is not None:
                     JSNumber.validate_int_bounds(step, "`step`")  # type: ignore
                 JSNumber.validate_int_bounds(value, "`value`")  # type: ignore
@@ -250,14 +317,13 @@ class NumberInputMixin:
         except JSNumberBoundsException as e:
             raise StreamlitAPIException(str(e))
 
+        data_type = NumberInputProto.INT if all_ints else NumberInputProto.FLOAT
+
         number_input_proto = NumberInputProto()
-        number_input_proto.data_type = (
-            NumberInputProto.INT if all_ints else NumberInputProto.FLOAT
-        )
+        number_input_proto.data_type = data_type
         number_input_proto.label = label
         number_input_proto.default = value
         number_input_proto.form_id = current_form_id(self.dg)
-        number_input_proto.disabled = disabled
         if help is not None:
             number_input_proto.help = dedent(help)
 
@@ -275,27 +341,32 @@ class NumberInputMixin:
         if format is not None:
             number_input_proto.format = format
 
-        def deserialize_number_input(ui_value, widget_id=""):
-            return ui_value if ui_value is not None else value
-
-        current_value, set_frontend_value = register_widget(
+        serde = NumberInputSerde(value, data_type)
+        widget_state = register_widget(
             "number_input",
             number_input_proto,
             user_key=key,
             on_change_handler=on_change,
             args=args,
             kwargs=kwargs,
-            deserializer=deserialize_number_input,
-            serializer=lambda x: x,
+            deserializer=serde.deserialize,
+            serializer=serde.serialize,
             ctx=ctx,
         )
 
-        if set_frontend_value:
-            number_input_proto.value = current_value
+        # This needs to be done after register_widget because we don't want
+        # the following proto fields to affect a widget's ID.
+        number_input_proto.disabled = disabled
+        number_input_proto.label_visibility.value = get_label_visibility_proto_value(
+            label_visibility
+        )
+
+        if widget_state.value_changed:
+            number_input_proto.value = widget_state.value
             number_input_proto.set_value = True
 
         self.dg._enqueue("number_input", number_input_proto)
-        return cast(Number, current_value)
+        return widget_state.value
 
     @property
     def dg(self) -> "streamlit.delta_generator.DeltaGenerator":
