@@ -1,10 +1,10 @@
-# Copyright 2018-2022 Streamlit Inc.
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#    http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
@@ -12,28 +12,61 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from streamlit.runtime.scriptrunner import ScriptRunContext, get_script_run_ctx
-from streamlit.type_util import Key, to_key
+from dataclasses import dataclass
 from textwrap import dedent
 from typing import Optional, cast
 
 import streamlit
+from streamlit.elements.form import current_form_id
+from streamlit.elements.utils import (
+    check_callback_rules,
+    check_session_state_rules,
+    get_label_visibility_proto_value,
+)
 from streamlit.errors import StreamlitAPIException
 from streamlit.proto.TextArea_pb2 import TextArea as TextAreaProto
 from streamlit.proto.TextInput_pb2 import TextInput as TextInputProto
+from streamlit.runtime.metrics_util import gather_metrics
+from streamlit.runtime.scriptrunner import ScriptRunContext, get_script_run_ctx
 from streamlit.runtime.state import (
-    register_widget,
     WidgetArgs,
     WidgetCallback,
     WidgetKwargs,
+    register_widget,
+)
+from streamlit.type_util import (
+    Key,
+    LabelVisibility,
+    SupportsStr,
+    maybe_raise_label_warnings,
+    to_key,
 )
 
-from .form import current_form_id
-from .utils import check_callback_rules, check_session_state_rules
-from ..type_util import SupportsStr
+
+@dataclass
+class TextInputSerde:
+    value: SupportsStr
+
+    def deserialize(self, ui_value: Optional[str], widget_id: str = "") -> str:
+        return str(ui_value if ui_value is not None else self.value)
+
+    def serialize(self, v: str) -> str:
+        return v
+
+
+@dataclass
+class TextAreaSerde:
+    value: SupportsStr
+
+    def deserialize(self, ui_value: Optional[str], widget_id: str = "") -> str:
+        return str(ui_value if ui_value is not None else self.value)
+
+    def serialize(self, v: str) -> str:
+        return v
 
 
 class TextWidgetsMixin:
+    @gather_metrics("text_input")
     def text_input(
         self,
         label: str,
@@ -49,13 +82,38 @@ class TextWidgetsMixin:
         *,  # keyword-only arguments:
         placeholder: Optional[str] = None,
         disabled: bool = False,
+        label_visibility: LabelVisibility = "visible",
     ) -> str:
-        """Display a single-line text input widget.
+        r"""Display a single-line text input widget.
 
         Parameters
         ----------
         label : str
             A short label explaining to the user what this input is for.
+            The label can optionally contain Markdown and supports the following
+            elements: Bold, Italics, Strikethroughs, Inline Code, Emojis, and Links.
+
+            This also supports:
+
+            * Emoji shortcodes, such as ``:+1:``  and ``:sunglasses:``.
+              For a list of all supported codes,
+              see https://share.streamlit.io/streamlit/emoji-shortcodes.
+
+            * LaTeX expressions, by wrapping them in "$" or "$$" (the "$$"
+              must be on their own lines). Supported LaTeX functions are listed
+              at https://katex.org/docs/supported.html.
+
+            * Colored text, using the syntax ``:color[text to be colored]``,
+              where ``color`` needs to be replaced with any of the following
+              supported colors: blue, green, orange, red, violet.
+
+            Unsupported elements are not displayed. Display unsupported elements
+            as literal characters by backslash-escaping them. E.g.
+            ``1\. Not an ordered list``.
+
+            For accessibility reasons, you should never set an empty label (label="")
+            but hide it with label_visibility if needed. In the future, we may disallow
+            empty labels by raising an exception.
         value : object
             The text value of this widget when it first renders. This will be
             cast to str internally.
@@ -89,6 +147,11 @@ class TextWidgetsMixin:
         disabled : bool
             An optional boolean, which disables the text input if set to True.
             The default is False. This argument can only be supplied by keyword.
+        label_visibility : "visible" or "hidden" or "collapsed"
+            The visibility of the label. If "hidden", the label doesn't show but there
+            is still empty space for it above the widget (equivalent to label="").
+            If "collapsed", both the label and the space are removed. Default is
+            "visible". This argument can only be supplied by keyword.
 
         Returns
         -------
@@ -97,6 +160,8 @@ class TextWidgetsMixin:
 
         Example
         -------
+        >>> import streamlit as st
+        >>>
         >>> title = st.text_input('Movie title', 'Life of Brian')
         >>> st.write('The current movie title is', title)
 
@@ -119,6 +184,7 @@ class TextWidgetsMixin:
             kwargs=kwargs,
             placeholder=placeholder,
             disabled=disabled,
+            label_visibility=label_visibility,
             ctx=ctx,
         )
 
@@ -137,12 +203,15 @@ class TextWidgetsMixin:
         *,  # keyword-only arguments:
         placeholder: Optional[str] = None,
         disabled: bool = False,
+        label_visibility: LabelVisibility = "visible",
         ctx: Optional[ScriptRunContext] = None,
     ) -> str:
         key = to_key(key)
         check_callback_rules(self.dg, on_change)
         check_session_state_rules(default_value=None if value == "" else value, key=key)
 
+        maybe_raise_label_warnings(label, label_visibility)
+
         text_input_proto = TextInputProto()
         text_input_proto.label = label
         text_input_proto.default = str(value)
@@ -173,8 +242,7 @@ class TextWidgetsMixin:
             autocomplete = "new-password" if type == "password" else ""
         text_input_proto.autocomplete = autocomplete
 
-        def deserialize_text_input(ui_value: Optional[str], widget_id: str = "") -> str:
-            return str(ui_value if ui_value is not None else value)
+        serde = TextInputSerde(value)
 
         widget_state = register_widget(
             "text_input",
@@ -183,14 +251,17 @@ class TextWidgetsMixin:
             on_change_handler=on_change,
             args=args,
             kwargs=kwargs,
-            deserializer=deserialize_text_input,
-            serializer=lambda x: x,
+            deserializer=serde.deserialize,
+            serializer=serde.serialize,
             ctx=ctx,
         )
 
         # This needs to be done after register_widget because we don't want
         # the following proto fields to affect a widget's ID.
         text_input_proto.disabled = disabled
+        text_input_proto.label_visibility.value = get_label_visibility_proto_value(
+            label_visibility
+        )
         if widget_state.value_changed:
             text_input_proto.value = widget_state.value
             text_input_proto.set_value = True
@@ -198,6 +269,7 @@ class TextWidgetsMixin:
         self.dg._enqueue("text_input", text_input_proto)
         return widget_state.value
 
+    @gather_metrics("text_area")
     def text_area(
         self,
         label: str,
@@ -212,13 +284,38 @@ class TextWidgetsMixin:
         *,  # keyword-only arguments:
         placeholder: Optional[str] = None,
         disabled: bool = False,
+        label_visibility: LabelVisibility = "visible",
     ) -> str:
-        """Display a multi-line text input widget.
+        r"""Display a multi-line text input widget.
 
         Parameters
         ----------
         label : str
             A short label explaining to the user what this input is for.
+            The label can optionally contain Markdown and supports the following
+            elements: Bold, Italics, Strikethroughs, Inline Code, Emojis, and Links.
+
+            This also supports:
+
+            * Emoji shortcodes, such as ``:+1:``  and ``:sunglasses:``.
+              For a list of all supported codes,
+              see https://share.streamlit.io/streamlit/emoji-shortcodes.
+
+            * LaTeX expressions, by wrapping them in "$" or "$$" (the "$$"
+              must be on their own lines). Supported LaTeX functions are listed
+              at https://katex.org/docs/supported.html.
+
+            * Colored text, using the syntax ``:color[text to be colored]``,
+              where ``color`` needs to be replaced with any of the following
+              supported colors: blue, green, orange, red, violet.
+
+            Unsupported elements are not displayed. Display unsupported elements
+            as literal characters by backslash-escaping them. E.g.
+            ``1\. Not an ordered list``.
+
+            For accessibility reasons, you should never set an empty label (label="")
+            but hide it with label_visibility if needed. In the future, we may disallow
+            empty labels by raising an exception.
         value : object
             The text value of this widget when it first renders. This will be
             cast to str internally.
@@ -246,6 +343,11 @@ class TextWidgetsMixin:
         disabled : bool
             An optional boolean, which disables the text area if set to True.
             The default is False. This argument can only be supplied by keyword.
+        label_visibility : "visible" or "hidden" or "collapsed"
+            The visibility of the label. If "hidden", the label doesn't show but there
+            is still empty space for it above the widget (equivalent to label="").
+            If "collapsed", both the label and the space are removed. Default is
+            "visible". This argument can only be supplied by keyword.
 
         Returns
         -------
@@ -254,6 +356,8 @@ class TextWidgetsMixin:
 
         Example
         -------
+        >>> import streamlit as st
+        >>>
         >>> txt = st.text_area('Text to analyze', '''
         ...     It was the best of times, it was the worst of times, it was
         ...     the age of wisdom, it was the age of foolishness, it was
@@ -277,6 +381,7 @@ class TextWidgetsMixin:
             kwargs=kwargs,
             placeholder=placeholder,
             disabled=disabled,
+            label_visibility=label_visibility,
             ctx=ctx,
         )
 
@@ -294,12 +399,15 @@ class TextWidgetsMixin:
         *,  # keyword-only arguments:
         placeholder: Optional[str] = None,
         disabled: bool = False,
+        label_visibility: LabelVisibility = "visible",
         ctx: Optional[ScriptRunContext] = None,
     ) -> str:
         key = to_key(key)
         check_callback_rules(self.dg, on_change)
         check_session_state_rules(default_value=None if value == "" else value, key=key)
 
+        maybe_raise_label_warnings(label, label_visibility)
+
         text_area_proto = TextAreaProto()
         text_area_proto.label = label
         text_area_proto.default = str(value)
@@ -317,9 +425,7 @@ class TextWidgetsMixin:
         if placeholder is not None:
             text_area_proto.placeholder = str(placeholder)
 
-        def deserialize_text_area(ui_value, widget_id="") -> str:
-            return str(ui_value if ui_value is not None else value)
-
+        serde = TextAreaSerde(value)
         widget_state = register_widget(
             "text_area",
             text_area_proto,
@@ -327,14 +433,17 @@ class TextWidgetsMixin:
             on_change_handler=on_change,
             args=args,
             kwargs=kwargs,
-            deserializer=deserialize_text_area,
-            serializer=lambda x: x,
+            deserializer=serde.deserialize,
+            serializer=serde.serialize,
             ctx=ctx,
         )
 
         # This needs to be done after register_widget because we don't want
         # the following proto fields to affect a widget's ID.
         text_area_proto.disabled = disabled
+        text_area_proto.label_visibility.value = get_label_visibility_proto_value(
+            label_visibility
+        )
         if widget_state.value_changed:
             text_area_proto.value = widget_state.value
             text_area_proto.set_value = True
