@@ -1,10 +1,10 @@
-# Copyright 2018-2022 Streamlit Inc.
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#    http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
@@ -12,14 +12,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import textwrap
-from typing import Any, cast, Hashable, Optional, TYPE_CHECKING, Union
+from typing import TYPE_CHECKING, Any, Hashable, Optional, Union, cast
 
 import streamlit
-from streamlit import type_util
+from streamlit import runtime, type_util
 from streamlit.elements.form import is_in_form
 from streamlit.errors import StreamlitAPIException
-from streamlit.runtime.state import get_session_state, WidgetCallback
+from streamlit.proto.LabelVisibilityMessage_pb2 import LabelVisibilityMessage
+from streamlit.runtime.state import WidgetCallback, get_session_state
 
 if TYPE_CHECKING:
     from streamlit.delta_generator import DeltaGenerator
@@ -41,11 +41,7 @@ def last_index_for_melted_dataframes(
 def check_callback_rules(
     dg: "DeltaGenerator", on_change: Optional[WidgetCallback]
 ) -> None:
-    if (
-        streamlit._is_running_with_streamlit
-        and is_in_form(dg)
-        and on_change is not None
-    ):
+    if runtime.exists() and is_in_form(dg) and on_change is not None:
         raise StreamlitAPIException(
             "With forms, callbacks can only be defined on the `st.form_submit_button`."
             " Defining callbacks on other widgets inside a form is not allowed."
@@ -60,7 +56,7 @@ def check_session_state_rules(
 ) -> None:
     global _shown_default_value_warning
 
-    if key is None or not streamlit._is_running_with_streamlit:
+    if key is None or not runtime.exists():
         return
 
     session_state = get_session_state()
@@ -79,3 +75,18 @@ def check_session_state_rules(
             " also had its value set via the Session State API."
         )
         _shown_default_value_warning = True
+
+
+def get_label_visibility_proto_value(
+    label_visibility_string: type_util.LabelVisibility,
+) -> "LabelVisibilityMessage.LabelVisibilityOptions.ValueType":
+    """Returns one of LabelVisibilityMessage enum constants.py based on string value."""
+
+    if label_visibility_string == "visible":
+        return LabelVisibilityMessage.LabelVisibilityOptions.VISIBLE
+    elif label_visibility_string == "hidden":
+        return LabelVisibilityMessage.LabelVisibilityOptions.HIDDEN
+    elif label_visibility_string == "collapsed":
+        return LabelVisibilityMessage.LabelVisibilityOptions.COLLAPSED
+
+    raise ValueError(f"Unknown label visibility value: {label_visibility_string}")
