@@ -1,26 +1,24 @@
-# Copyright 2018-2022 Streamlit Inc.
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#    http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 import asyncio
 import sys
 import uuid
-from asyncio import AbstractEventLoop
 from enum import Enum
-from typing import TYPE_CHECKING, Callable, Dict, Optional, List, Union
+from typing import TYPE_CHECKING, Callable, Dict, List, Optional, Union
 
 import streamlit.elements.exception as exception_utils
-from streamlit import __version__, config, source_util, secrets
+from streamlit import config, runtime, source_util
 from streamlit.case_converters import to_snake_case
 from streamlit.logger import get_logger
 from streamlit.proto.BackMsg_pb2 import BackMsg
@@ -34,22 +32,20 @@ from streamlit.proto.NewSession_pb2 import (
     UserInfo,
 )
 from streamlit.proto.PagesChanged_pb2 import PagesChanged
+from streamlit.runtime import caching, legacy_caching
+from streamlit.runtime.credentials import Credentials
+from streamlit.runtime.forward_msg_queue import ForwardMsgQueue
+from streamlit.runtime.metrics_util import Installation
+from streamlit.runtime.script_data import ScriptData
+from streamlit.runtime.scriptrunner import RerunData, ScriptRunner, ScriptRunnerEvent
+from streamlit.runtime.secrets import secrets_singleton
+from streamlit.runtime.uploaded_file_manager import UploadedFileManager
+from streamlit.version import STREAMLIT_VERSION_STRING
 from streamlit.watcher import LocalSourcesWatcher
-from . import caching, legacy_caching
-from .credentials import Credentials
-from .in_memory_file_manager import in_memory_file_manager
-from .metrics_util import Installation
-from .scriptrunner import (
-    RerunData,
-    ScriptRunner,
-    ScriptRunnerEvent,
-)
-from .session_data import SessionData
-from .uploaded_file_manager import UploadedFileManager
 
 LOGGER = get_logger(__name__)
 if TYPE_CHECKING:
-    from .state import SessionState
+    from streamlit.runtime.state import SessionState
 
 
 class AppSessionState(Enum):
@@ -68,7 +64,7 @@ class AppSession:
     Contains session data for a single "user" of an active app
     (that is, a connected browser tab).
 
-    Each AppSession has its own SessionData, root DeltaGenerator, ScriptRunner,
+    Each AppSession has its own ScriptData, root DeltaGenerator, ScriptRunner,
     and widget state.
 
     An AppSession is attached to each thread involved in running its script.
@@ -77,25 +73,21 @@ class AppSession:
 
     def __init__(
         self,
-        event_loop: AbstractEventLoop,
-        session_data: SessionData,
+        script_data: ScriptData,
         uploaded_file_manager: UploadedFileManager,
         message_enqueued_callback: Optional[Callable[[], None]],
         local_sources_watcher: LocalSourcesWatcher,
         user_info: Dict[str, Optional[str]],
-    ):
+    ) -> None:
         """Initialize the AppSession.
 
         Parameters
         ----------
-        event_loop : AbstractEventLoop
-            The asyncio EventLoop that we're running on.
-
-        session_data : SessionData
+        script_data : ScriptData
             Object storing parameters related to running a script
 
         uploaded_file_manager : UploadedFileManager
-            The server's UploadedFileManager.
+            Used to manage files uploaded by users via the Streamlit web client.
 
         message_enqueued_callback : Callable[[], None]
             After enqueuing a message, this callable notification will be invoked.
@@ -118,9 +110,14 @@ class AppSession:
         # Each AppSession has a unique string ID.
         self.id = str(uuid.uuid4())
 
-        self._event_loop = event_loop
-        self._session_data = session_data
+        self._event_loop = asyncio.get_running_loop()
+        self._script_data = script_data
         self._uploaded_file_mgr = uploaded_file_manager
+
+        # The browser queue contains messages that haven't yet been
+        # delivered to the browser. Periodically, the server flushes
+        # this queue and delivers its contents to the browser.
+        self._browser_queue = ForwardMsgQueue()
         self._message_enqueued_callback = message_enqueued_callback
 
         self._state = AppSessionState.APP_NOT_RUNNING
@@ -129,19 +126,13 @@ class AppSession:
         # due to the source code changing we need to pass in the previous client state.
         self._client_state = ClientState()
 
-        self._local_sources_watcher = local_sources_watcher
-        self._local_sources_watcher.register_file_change_callback(
-            self._on_source_file_changed
-        )
-        self._stop_config_listener = config.on_config_parsed(
-            self._on_source_file_changed, force_connect=True
-        )
-        self._stop_pages_listener = source_util.register_pages_changed_callback(
-            self._on_pages_changed
-        )
+        self._local_sources_watcher: Optional[
+            LocalSourcesWatcher
+        ] = local_sources_watcher
+        self._stop_config_listener: Optional[Callable[[], bool]] = None
+        self._stop_pages_listener: Optional[Callable[[], bool]] = None
 
-        # The script should rerun when the `secrets.toml` file has been changed.
-        secrets._file_change_listener.connect(self._on_secrets_file_changed)
+        self.register_file_watchers()
 
         self._run_on_save = config.get_option("server.runOnSave")
 
@@ -157,6 +148,57 @@ class AppSession:
 
         LOGGER.debug("AppSession initialized (id=%s)", self.id)
 
+    def __del__(self) -> None:
+        """Ensure that we call shutdown() when an AppSession is garbage collected."""
+        self.shutdown()
+
+    def register_file_watchers(self) -> None:
+        """Register handlers to be called when various files are changed.
+
+        Files that we watch include:
+          * source files that already exist (for edits)
+          * `.py` files in the the main script's `pages/` directory (for file additions
+            and deletions)
+          * project and user-level config.toml files
+          * the project-level secrets.toml files
+
+        This method is called automatically on AppSession construction, but it may be
+        called again in the case when a session is disconnected and is being reconnect
+        to.
+        """
+        if self._local_sources_watcher is None:
+            self._local_sources_watcher = LocalSourcesWatcher(
+                self._script_data.main_script_path
+            )
+
+        self._local_sources_watcher.register_file_change_callback(
+            self._on_source_file_changed
+        )
+        self._stop_config_listener = config.on_config_parsed(
+            self._on_source_file_changed, force_connect=True
+        )
+        self._stop_pages_listener = source_util.register_pages_changed_callback(
+            self._on_pages_changed
+        )
+        secrets_singleton._file_change_listener.connect(self._on_secrets_file_changed)
+
+    def disconnect_file_watchers(self) -> None:
+        """Disconnect the file watcher handlers registered by register_file_watchers."""
+        if self._local_sources_watcher is not None:
+            self._local_sources_watcher.close()
+        if self._stop_config_listener is not None:
+            self._stop_config_listener()
+        if self._stop_pages_listener is not None:
+            self._stop_pages_listener()
+
+        secrets_singleton._file_change_listener.disconnect(
+            self._on_secrets_file_changed
+        )
+
+        self._local_sources_watcher = None
+        self._stop_config_listener = None
+        self._stop_pages_listener = None
+
     def flush_browser_queue(self) -> List[ForwardMsg]:
         """Clear the forward message queue and return the messages it contained.
 
@@ -170,7 +212,7 @@ class AppSession:
             be delivered to the browser.
 
         """
-        return self._session_data.flush_browser_queue()
+        return self._browser_queue.flush()
 
     def shutdown(self) -> None:
         """Shut down the AppSession.
@@ -183,8 +225,10 @@ class AppSession:
             # Clear any unused session files in upload file manager and media
             # file manager
             self._uploaded_file_mgr.remove_session_files(self.id)
-            in_memory_file_manager.clear_session_files(self.id)
-            in_memory_file_manager.del_expired_files()
+
+            if runtime.exists():
+                runtime.get_instance().media_file_mgr.clear_session_refs(self.id)
+                runtime.get_instance().media_file_mgr.remove_orphaned_files()
 
             # Shut down the ScriptRunner, if one is active.
             # self._state must not be set to SHUTDOWN_REQUESTED until
@@ -193,12 +237,10 @@ class AppSession:
                 self._scriptrunner.request_stop()
 
             self._state = AppSessionState.SHUTDOWN_REQUESTED
-            self._local_sources_watcher.close()
-            if self._stop_config_listener is not None:
-                self._stop_config_listener()
-            if self._stop_pages_listener is not None:
-                self._stop_pages_listener()
-            secrets._file_change_listener.disconnect(self._on_secrets_file_changed)
+
+            # Disconnect all file watchers if we haven't already, although we will have
+            # generally already done so by the time we get here.
+            self.disconnect_file_watchers()
 
     def _enqueue_forward_msg(self, msg: ForwardMsg) -> None:
         """Enqueue a new ForwardMsg to our browser queue.
@@ -218,7 +260,7 @@ class AppSession:
         if self._debug_last_backmsg_id:
             msg.debug_last_backmsg_id = self._debug_last_backmsg_id
 
-        self._session_data.enqueue(msg)
+        self._browser_queue.enqueue(msg)
         if self._message_enqueued_callback:
             self._message_enqueued_callback()
 
@@ -243,9 +285,9 @@ class AppSession:
             else:
                 LOGGER.warning('No handler for "%s"', msg_type)
 
-        except BaseException as e:
-            LOGGER.error(e)
-            self.handle_backmsg_exception(e)
+        except Exception as ex:
+            LOGGER.error(ex)
+            self.handle_backmsg_exception(ex)
 
     def handle_backmsg_exception(self, e: BaseException) -> None:
         """Handle an Exception raised while processing a BackMsg from the browser."""
@@ -327,7 +369,7 @@ class AppSession:
         """Create and run a new ScriptRunner with the given RerunData."""
         self._scriptrunner = ScriptRunner(
             session_id=self.id,
-            main_script_path=self._session_data.main_script_path,
+            main_script_path=self._script_data.main_script_path,
             client_state=self._client_state,
             session_state=self._session_state,
             uploaded_file_mgr=self._uploaded_file_mgr,
@@ -342,7 +384,7 @@ class AppSession:
         return self._session_state
 
     def _should_rerun_on_file_change(self, filepath: str) -> bool:
-        main_script_path = self._session_data.main_script_path
+        main_script_path = self._script_data.main_script_path
         pages = source_util.get_pages(main_script_path)
 
         changed_page_script_hash = next(
@@ -377,17 +419,12 @@ class AppSession:
         self._on_source_file_changed()
 
     def _on_pages_changed(self, _) -> None:
-        # TODO: Double-check the product behavior we want on this. In the spec,
-        # it says that we should notify the client of a pages dir change only
-        # if "run on save" is true, but I feel like always sending updates is
-        # quite reasonable behavior since the pages nav updating is not
-        # potentially disruptive like a script rerunning is.
         msg = ForwardMsg()
-        _populate_app_pages(msg.pages_changed, self._session_data.main_script_path)
+        _populate_app_pages(msg.pages_changed, self._script_data.main_script_path)
         self._enqueue_forward_msg(msg)
 
     def _clear_queue(self) -> None:
-        self._session_data.clear_browser_queue()
+        self._browser_queue.clear()
 
     def _on_scriptrunner_event(
         self,
@@ -452,7 +489,7 @@ class AppSession:
 
         assert (
             self._event_loop == asyncio.get_running_loop()
-        ), "This function must only be called on the eventloop thread"
+        ), "This function must only be called on the eventloop thread the AppSession was created on."
 
         if sender is not self._scriptrunner:
             # This event was sent by a non-current ScriptRunner; ignore it.
@@ -501,7 +538,8 @@ class AppSession:
                 # The script completed successfully: update our
                 # LocalSourcesWatcher to account for any source code changes
                 # that change which modules should be watched.
-                self._local_sources_watcher.update_watched_modules()
+                if self._local_sources_watcher:
+                    self._local_sources_watcher.update_watched_modules()
             else:
                 # The script didn't complete successfully: send the exception
                 # to the frontend.
@@ -519,7 +557,8 @@ class AppSession:
                 ForwardMsg.FINISHED_EARLY_FOR_RERUN
             )
             self._enqueue_forward_msg(script_finished_msg)
-            self._local_sources_watcher.update_watched_modules()
+            if self._local_sources_watcher:
+                self._local_sources_watcher.update_watched_modules()
 
         elif event == ScriptRunnerEvent.SHUTDOWN:
             assert (
@@ -529,7 +568,7 @@ class AppSession:
             if self._state == AppSessionState.SHUTDOWN_REQUESTED:
                 # Only clear media files if the script is done running AND the
                 # session is actually shutting down.
-                in_memory_file_manager.clear_session_files(self.id)
+                runtime.get_instance().media_file_mgr.clear_session_refs(self.id)
 
             self._client_state = client_state
             self._scriptrunner = None
@@ -544,13 +583,13 @@ class AppSession:
         app_was_running = prev_state == AppSessionState.APP_IS_RUNNING
         app_is_running = self._state == AppSessionState.APP_IS_RUNNING
         if app_is_running != app_was_running:
-            self._enqueue_forward_msg(self._create_session_state_changed_message())
+            self._enqueue_forward_msg(self._create_session_status_changed_message())
 
-    def _create_session_state_changed_message(self) -> ForwardMsg:
-        """Create and return a session_state_changed ForwardMsg."""
+    def _create_session_status_changed_message(self) -> ForwardMsg:
+        """Create and return a session_status_changed ForwardMsg."""
         msg = ForwardMsg()
-        msg.session_state_changed.run_on_save = self._run_on_save
-        msg.session_state_changed.script_is_running = (
+        msg.session_status_changed.run_on_save = self._run_on_save
+        msg.session_status_changed.script_is_running = (
             self._state == AppSessionState.APP_IS_RUNNING
         )
         return msg
@@ -566,11 +605,11 @@ class AppSession:
         msg = ForwardMsg()
 
         msg.new_session.script_run_id = _generate_scriptrun_id()
-        msg.new_session.name = self._session_data.name
-        msg.new_session.main_script_path = self._session_data.main_script_path
+        msg.new_session.name = self._script_data.name
+        msg.new_session.main_script_path = self._script_data.main_script_path
         msg.new_session.page_script_hash = page_script_hash
 
-        _populate_app_pages(msg.new_session, self._session_data.main_script_path)
+        _populate_app_pages(msg.new_session, self._script_data.main_script_path)
         _populate_config_msg(msg.new_session.config)
         _populate_theme_msg(msg.new_session.custom_theme)
 
@@ -582,15 +621,15 @@ class AppSession:
 
         _populate_user_info_msg(imsg.user_info)
 
-        imsg.environment_info.streamlit_version = __version__
+        imsg.environment_info.streamlit_version = STREAMLIT_VERSION_STRING
         imsg.environment_info.python_version = ".".join(map(str, sys.version_info))
 
-        imsg.session_state.run_on_save = self._run_on_save
-        imsg.session_state.script_is_running = (
+        imsg.session_status.run_on_save = self._run_on_save
+        imsg.session_status.script_is_running = (
             self._state == AppSessionState.APP_IS_RUNNING
         )
 
-        imsg.command_line = self._session_data.command_line
+        imsg.command_line = self._script_data.command_line
         imsg.session_id = self.id
 
         return msg
@@ -615,7 +654,7 @@ class AppSession:
         try:
             from streamlit.git_util import GitRepo
 
-            repo = GitRepo(self._session_data.main_script_path)
+            repo = GitRepo(self._script_data.main_script_path)
 
             repo_info = repo.get_repo_info()
             if repo_info is None:
@@ -638,10 +677,10 @@ class AppSession:
                 msg.git_info_changed.state = GitInfo.GitStates.DEFAULT
 
             self._enqueue_forward_msg(msg)
-        except Exception as e:
+        except Exception as ex:
             # Users may never even install Git in the first place, so this
             # error requires no action. It can be useful for debugging.
-            LOGGER.debug("Obtaining Git information produced an error", exc_info=e)
+            LOGGER.debug("Obtaining Git information produced an error", exc_info=ex)
 
     def _handle_rerun_script_request(
         self, client_state: Optional[ClientState] = None
@@ -669,8 +708,8 @@ class AppSession:
 
         """
         legacy_caching.clear_cache()
-        caching.memo.clear()
-        caching.singleton.clear()
+        caching.cache_data.clear()
+        caching.cache_resource.clear()
         self._session_state.clear()
 
     def _handle_set_run_on_save_request(self, new_value: bool) -> None:
@@ -685,7 +724,7 @@ class AppSession:
 
         """
         self._run_on_save = new_value
-        self._enqueue_forward_msg(self._create_session_state_changed_message())
+        self._enqueue_forward_msg(self._create_session_status_changed_message())
 
 
 def _populate_config_msg(msg: Config) -> None:
