@@ -1,10 +1,10 @@
-# Copyright 2018-2022 Streamlit Inc.
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#    http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
@@ -18,9 +18,8 @@ import functools
 import hashlib
 import os
 import subprocess
-import numpy as np
-
 from typing import Any, Dict, Iterable, List, Mapping, TypeVar
+
 from typing_extensions import Final
 
 from streamlit import env_util
@@ -32,7 +31,7 @@ FLOAT_EQUALITY_EPSILON: Final[float] = 0.000000000005
 
 def memoize(func):
     """Decorator to memoize the result of a no-args func."""
-    result = []  # type: List[Any]
+    result: List[Any] = []
 
     @functools.wraps(func)
     def wrapped_func():
@@ -55,7 +54,6 @@ def open_browser(url):
         The URL. Must include the protocol.
 
     """
-
     # Treat Windows separately because:
     # 1. /dev/null doesn't exist.
     # 2. subprocess.Popen(['start', url]) doesn't actually pop up the
@@ -130,13 +128,12 @@ def index_(iterable: Iterable[_Value], x: _Value) -> int:
     -------
     int
     """
-
     for i, value in enumerate(iterable):
-        if isinstance(value, float) and isinstance(x, float):
+        if x == value:
+            return i
+        elif isinstance(value, float) and isinstance(x, float):
             if abs(x - value) < FLOAT_EQUALITY_EPSILON:
                 return i
-        elif x == value:
-            return i
     raise ValueError("{} is not in iterable".format(str(x)))
 
 
