@@ -1,10 +1,10 @@
-# Copyright 2018-2022 Streamlit Inc.
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#    http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
@@ -12,23 +12,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import json
 import os
-from urllib.parse import quote, unquote_plus
 
 import tornado.web
 
 from streamlit import config, file_util
 from streamlit.logger import get_logger
-from streamlit.runtime.in_memory_file_manager import (
-    _get_extension_for_mimetype,
-    in_memory_file_manager,
-    FILE_TYPE_DOWNLOADABLE,
-)
 from streamlit.runtime.runtime_util import serialize_forward_msg
-from streamlit.string_util import generate_download_filename_from_title
+from streamlit.web.server.server_util import emit_endpoint_deprecation_notice
 
-LOGGER = get_logger(__name__)
+_LOGGER = get_logger(__name__)
 
 
 def allow_cross_origin_requests():
@@ -107,106 +100,6 @@ class AddSlashHandler(tornado.web.RequestHandler):
         pass
 
 
-class MediaFileHandler(tornado.web.StaticFileHandler):
-    def set_default_headers(self):
-        if allow_cross_origin_requests():
-            self.set_header("Access-Control-Allow-Origin", "*")
-
-    def set_extra_headers(self, path: str) -> None:
-        """Add Content-Disposition header for downloadable files.
-
-        Set header value to "attachment" indicating that file should be saved
-        locally instead of displaying inline in browser.
-
-        We also set filename to specify filename for  downloaded file.
-        Used for serve downloadable files, like files stored
-        via st.download_button widget
-        """
-        in_memory_file = in_memory_file_manager.get(path)
-
-        if in_memory_file and in_memory_file.file_type == FILE_TYPE_DOWNLOADABLE:
-            file_name = in_memory_file.file_name
-
-            if not file_name:
-                title = self.get_argument("title", "", True)
-                title = unquote_plus(title)
-                filename = generate_download_filename_from_title(title)
-                file_name = (
-                    f"{filename}{_get_extension_for_mimetype(in_memory_file.mimetype)}"
-                )
-
-            try:
-                file_name.encode("ascii")
-                file_expr = 'filename="{}"'.format(file_name)
-            except UnicodeEncodeError:
-                file_expr = "filename*=utf-8''{}".format(quote(file_name))
-
-            self.set_header("Content-Disposition", f"attachment; {file_expr}")
-
-    # Overriding StaticFileHandler to use the InMemoryFileManager
-    #
-    # From the Torndado docs:
-    # To replace all interaction with the filesystem (e.g. to serve
-    # static content from a database), override `get_content`,
-    # `get_content_size`, `get_modified_time`, `get_absolute_path`, and
-    # `validate_absolute_path`.
-    def validate_absolute_path(self, root, absolute_path):
-        try:
-            in_memory_file_manager.get(absolute_path)
-        except KeyError:
-            LOGGER.error("InMemoryFileManager: Missing file %s" % absolute_path)
-            raise tornado.web.HTTPError(404, "not found")
-
-        return absolute_path
-
-    def get_content_size(self):
-        abspath = self.absolute_path
-        if abspath is None:
-            return 0
-
-        in_memory_file = in_memory_file_manager.get(abspath)
-        return in_memory_file.content_size
-
-    def get_modified_time(self):
-        # We do not track last modified time, but this can be improved to
-        # allow caching among files in the InMemoryFileManager
-        return None
-
-    @classmethod
-    def get_absolute_path(cls, root, path):
-        # All files are stored in memory, so the absolute path is just the
-        # path itself. In the MediaFileHandler, it's just the filename
-        return path
-
-    @classmethod
-    def get_content(cls, abspath, start=None, end=None):
-        LOGGER.debug("MediaFileHandler: GET %s" % abspath)
-
-        try:
-            # abspath is the hash as used `get_absolute_path`
-            in_memory_file = in_memory_file_manager.get(abspath)
-        except:
-            LOGGER.error("InMemoryFileManager: Missing file %s" % abspath)
-            return
-
-        LOGGER.debug(
-            "InMemoryFileManager: Sending %s file %s"
-            % (in_memory_file.mimetype, abspath)
-        )
-
-        # If there is no start and end, just return the full content
-        if start is None and end is None:
-            return in_memory_file.content
-
-        if start is None:
-            start = 0
-        if end is None:
-            end = len(in_memory_file.content)
-
-        # content is bytes that work just by slicing supplied by start and end
-        return in_memory_file.content[start:end]
-
-
 class _SpecialRequestHandler(tornado.web.RequestHandler):
     """Superclass for "special" endpoints, like /healthz."""
 
@@ -249,6 +142,14 @@ class HealthHandler(_SpecialRequestHandler):
         self._callback = callback
 
     async def get(self):
+        if self.request.uri and "_stcore/" not in self.request.uri:
+            new_path = (
+                "/_stcore/script-health-check"
+                if "script-health-check" in self.request.uri
+                else "/_stcore/health"
+            )
+            emit_endpoint_deprecation_notice(self, new_path=new_path)
+
         ok, msg = await self._callback()
         if ok:
             self.write(msg)
@@ -268,6 +169,46 @@ class HealthHandler(_SpecialRequestHandler):
             self.write(msg)
 
 
+# NOTE: We eventually want to get rid of this hard-coded list entirely as we don't want
+# to have links to Community Cloud live in the open source library in a way that affects
+# functionality (links advertising Community Cloud are probably okay 🙂). In the long
+# run, this list will most likely be replaced by a config option allowing us to more
+# granularly control what domains a Streamlit app should accept cross-origin iframe
+# messages from.
+ALLOWED_MESSAGE_ORIGINS = [
+    "https://devel.streamlit.test",
+    "https://share.streamlit.io",
+    "https://share-demo.streamlit.io",
+    "https://share-head.streamlit.io",
+    "https://share-staging.streamlit.io",
+    "https://*.demo.streamlit.run",
+    "https://*.head.streamlit.run",
+    "https://*.staging.streamlit.run",
+    "https://*.streamlitapp.test",
+    "https://*.streamlitapp.com",
+    "https://*.streamlit.run",
+    "https://*.demo.streamlit.app",
+    "https://*.head.streamlit.app",
+    "https://*.staging.streamlit.app",
+    "https://*.streamlit.app",
+]
+
+
+class AllowedMessageOriginsHandler(_SpecialRequestHandler):
+    async def get(self) -> None:
+        # ALLOWED_MESSAGE_ORIGINS must be wrapped in a dictionary because Tornado
+        # disallows writing lists directly into responses due to potential XSS
+        # vulnerabilities.
+        # See https://www.tornadoweb.org/en/stable/web.html#tornado.web.RequestHandler.write
+        self.write(
+            {
+                "allowedOrigins": ALLOWED_MESSAGE_ORIGINS,
+                "useExternalAuthToken": False,
+            }
+        )
+        self.set_status(200)
+
+
 class MessageCacheHandler(tornado.web.RequestHandler):
     """Returns ForwardMsgs from our MessageCache"""
 
@@ -289,8 +230,8 @@ class MessageCacheHandler(tornado.web.RequestHandler):
         msg_hash = self.get_argument("hash", None)
         if msg_hash is None:
             # Hash is missing! This is a malformed request.
-            LOGGER.error(
-                "HTTP request for cached message is " "missing the hash attribute."
+            _LOGGER.error(
+                "HTTP request for cached message is missing the hash attribute."
             )
             self.set_status(404)
             raise tornado.web.Finish()
@@ -298,14 +239,14 @@ class MessageCacheHandler(tornado.web.RequestHandler):
         message = self._cache.get_message(msg_hash)
         if message is None:
             # Message not in our cache.
-            LOGGER.error(
+            _LOGGER.error(
                 "HTTP request for cached message could not be fulfilled. "
-                "No such message: %s" % msg_hash
+                "No such message"
             )
             self.set_status(404)
             raise tornado.web.Finish()
 
-        LOGGER.debug("MessageCache HIT [hash=%s]" % msg_hash)
+        _LOGGER.debug("MessageCache HIT")
         msg_str = serialize_forward_msg(message)
         self.set_header("Content-Type", "application/octet-stream")
         self.write(msg_str)
