@@ -1,10 +1,10 @@
-# Copyright 2018-2022 Streamlit Inc.
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#    http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
@@ -17,7 +17,7 @@ import logging
 import os
 import socket
 import sys
-from typing import Any, Optional, List, Awaitable
+from typing import Any, Awaitable, List, Optional
 
 import click
 import tornado.concurrent
@@ -26,36 +26,33 @@ import tornado.netutil
 import tornado.web
 import tornado.websocket
 from tornado.httpserver import HTTPServer
+from typing_extensions import Final
 
-from streamlit import config
-from streamlit import file_util
-from streamlit import source_util
-from streamlit import util
+from streamlit import config, file_util, source_util, util
 from streamlit.components.v1.components import ComponentRegistry
 from streamlit.config_option import ConfigOption
 from streamlit.logger import get_logger
-from streamlit.runtime.runtime import (
-    Runtime,
-    RuntimeConfig,
-    RuntimeState,
-)
+from streamlit.runtime import Runtime, RuntimeConfig, RuntimeState
+from streamlit.runtime.memory_media_file_storage import MemoryMediaFileStorage
 from streamlit.runtime.runtime_util import get_max_message_size_bytes
+from streamlit.web.server.app_static_file_handler import AppStaticFileHandler
+from streamlit.web.server.browser_websocket_handler import BrowserWebSocketHandler
+from streamlit.web.server.component_request_handler import ComponentRequestHandler
+from streamlit.web.server.media_file_handler import MediaFileHandler
 from streamlit.web.server.routes import (
     AddSlashHandler,
+    AllowedMessageOriginsHandler,
     AssetsFileHandler,
     HealthHandler,
-    MediaFileHandler,
     MessageCacheHandler,
     StaticFileHandler,
 )
 from streamlit.web.server.server_util import make_url_path_regex
+from streamlit.web.server.stats_request_handler import StatsRequestHandler
 from streamlit.web.server.upload_file_request_handler import (
-    UploadFileRequestHandler,
     UPLOAD_FILE_ROUTE,
+    UploadFileRequestHandler,
 )
-from .browser_websocket_handler import BrowserWebSocketHandler
-from .component_request_handler import ComponentRequestHandler
-from .stats_request_handler import StatsRequestHandler
 
 LOGGER = get_logger(__name__)
 
@@ -81,6 +78,16 @@ MAX_PORT_SEARCH_RETRIES = 100
 # to an unix socket.
 UNIX_SOCKET_PREFIX = "unix://"
 
+MEDIA_ENDPOINT: Final = "/media"
+STREAM_ENDPOINT: Final = r"_stcore/stream"
+METRIC_ENDPOINT: Final = r"(?:st-metrics|_stcore/metrics)"
+MESSAGE_ENDPOINT: Final = r"_stcore/message"
+HEALTH_ENDPOINT: Final = r"(?:healthz|_stcore/health)"
+ALLOWED_MESSAGE_ORIGIN_ENDPOINT: Final = r"_stcore/allowed-message-origins"
+SCRIPT_HEALTH_CHECK_ENDPOINT: Final = (
+    r"(?:script-health-check|_stcore/script-health-check)"
+)
+
 
 class RetriesExceeded(Exception):
     pass
@@ -169,13 +176,20 @@ class Server:
 
         self._main_script_path = main_script_path
 
+        # Initialize MediaFileStorage and its associated endpoint
+        media_file_storage = MemoryMediaFileStorage(MEDIA_ENDPOINT)
+        MediaFileHandler.initialize_storage(media_file_storage)
+
         self._runtime = Runtime(
             RuntimeConfig(
                 script_path=main_script_path,
                 command_line=command_line,
+                media_file_storage=media_file_storage,
             ),
         )
 
+        self._runtime.stats_mgr.register_provider(media_file_storage)
+
     def __repr__(self) -> str:
         return util.repr_(self)
 
@@ -210,25 +224,29 @@ class Server:
 
         routes: List[Any] = [
             (
-                make_url_path_regex(base, "stream"),
+                make_url_path_regex(base, STREAM_ENDPOINT),
                 BrowserWebSocketHandler,
                 dict(runtime=self._runtime),
             ),
             (
-                make_url_path_regex(base, "healthz"),
+                make_url_path_regex(base, HEALTH_ENDPOINT),
                 HealthHandler,
                 dict(callback=lambda: self._runtime.is_ready_for_browser_connection),
             ),
             (
-                make_url_path_regex(base, "message"),
+                make_url_path_regex(base, MESSAGE_ENDPOINT),
                 MessageCacheHandler,
                 dict(cache=self._runtime.message_cache),
             ),
             (
-                make_url_path_regex(base, "st-metrics"),
+                make_url_path_regex(base, METRIC_ENDPOINT),
                 StatsRequestHandler,
                 dict(stats_manager=self._runtime.stats_mgr),
             ),
+            (
+                make_url_path_regex(base, ALLOWED_MESSAGE_ORIGIN_ENDPOINT),
+                AllowedMessageOriginsHandler,
+            ),
             (
                 make_url_path_regex(
                     base,
@@ -245,7 +263,11 @@ class Server:
                 AssetsFileHandler,
                 {"path": "%s/" % file_util.get_assets_dir()},
             ),
-            (make_url_path_regex(base, "media/(.*)"), MediaFileHandler, {"path": ""}),
+            (
+                make_url_path_regex(base, f"{MEDIA_ENDPOINT}/(.*)"),
+                MediaFileHandler,
+                {"path": ""},
+            ),
             (
                 make_url_path_regex(base, "component/(.*)"),
                 ComponentRequestHandler,
@@ -257,7 +279,7 @@ class Server:
             routes.extend(
                 [
                     (
-                        make_url_path_regex(base, "script-health-check"),
+                        make_url_path_regex(base, SCRIPT_HEALTH_CHECK_ENDPOINT),
                         HealthHandler,
                         dict(
                             callback=lambda: self._runtime.does_script_run_without_error()
@@ -266,6 +288,17 @@ class Server:
                 ]
             )
 
+        if config.get_option("server.enableStaticServing"):
+            routes.extend(
+                [
+                    (
+                        make_url_path_regex(base, "app/static/(.*)"),
+                        AppStaticFileHandler,
+                        {"path": file_util.get_app_static_dir(self.main_script_path)},
+                    ),
+                ]
+            )
+
         if config.get_option("global.developmentMode"):
             LOGGER.debug("Serving static content from the Node dev server")
         else:
